/*
Copyright © 2021 Pierre Ozoux <pierre@ozoux.net>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package cmd

import (
  "bufio"
  "fmt"
	"github.com/spf13/cobra"
  "os"
  "io"
  "text/template" 
	"github.com/spf13/viper"
)

type rocketChatConfig struct {
	ConfigureOAuth   bool
}

var config rocketChatConfig
var cfgFile string
var tmplFile string

var rootCmd = &cobra.Command{
	Use:   "libre-cli",
	Short: "A brief description of your application",
	Long: `A longer description that spans multiple lines and likely contains
examples and usage of using your application. For example:

Cobra is a CLI library for Go that empowers applications.
This application is a tool to generate the needed files
to quickly create a Cobra application.`,
  Run: func(cmd *cobra.Command, args []string) {
    if tmplFile == "-" {
      Render(os.Stdin)
    } else {
      file, _ := os.Open(tmplFile) 
      Render(file)
    }

    return
  },
}

func Render(reader io.Reader) {
  var file string

  scanner := bufio.NewScanner(reader)
  for scanner.Scan() {
    file += scanner.Text() + "\n"
  }
  tmpl, err := template.New("tmpl").Parse(file)
  if err != nil { panic(err) }
  tmpl.Execute(os.Stdout, config)
}

func Execute() {
	if err := rootCmd.Execute(); err != nil {
    fmt.Println(err)
		os.Exit(1)
	}
}

func init() {
  cobra.OnInitialize(initConfig)
	rootCmd.PersistentFlags().StringVar(&cfgFile, "config", "", "config file (default is ./config.yaml)")
  rootCmd.PersistentFlags().StringVarP(&tmplFile, "template", "t", "", "template file, or - for stdin")
}

// initConfig reads in config file and ENV variables if set.
func initConfig() {
	if cfgFile != "" {
		// Use config file from the flag.
		viper.SetConfigFile(cfgFile)
	} else {
		// Search config in home directory with name ".libre-cli" (without extension).
		viper.AddConfigPath(".")
		viper.SetConfigName("config")
	}

	viper.AutomaticEnv() // read in environment variables that match

	// If a config file is found, read it in.
	if err := viper.ReadInConfig(); err == nil {
    //fmt.Println("Using config file:", viper.ConfigFileUsed())
  }
  err := viper.Unmarshal(&config)
  if err != nil {
    fmt.Println("unable to decode into struct, %v", err)
  }
  //fmt.Println(config)
}
